;************************************************************************
;
;    Programm:     RS232 Routinen
;    Datum:        01.08.05 MM  Grundversion
;    Angespasst:   01.08.05 MM  ...
;
;    SYSTEM:       89C2051 MCS51  2KB code memory
;    SYSTEM-Info:  89c2051 @8.000MHz
;                  via serial port using on-chip data memory for saving timing
;
;************************************************************************
;
;
; Einzufgen ins Hauptprogramm:
; INCLUDE 	  RS232.asm    'Einbindung der RS232 Routinen
; RS232: RS232_INIT, RS232_SEND, RS232_GET, RS232_GET_BW, RS232_CRLF, RS232_SPACE, RS232_SEND_S
;
; Zur Verfgung stehende Routinen:
; 1. RS232_INIT -		Initiert die RS232 Schnittstelle.
; 2. RS232_SEND -		Sendet Byte im Akku
; 3. RS232_GET -      	Empfangt Byte > in Akku
; 4. RS232_GET_BW -     Empfangt Byte, Warten auf Zeichen > in Akku
; 5. RS232_CRLF -       Sendet CR LF
; 6. RS232_SPACE -      Sendet Space
; 7. RS232_Hex -        Sendet Accu als HEX
; 8. RS232_SEND_S -	Zeichenkette Senden, Nullterminiert
;
; Aufrufe Beispiel:
; - CALL RS232_Init		'UART RS2323 Initialisieren 9600BPS
;
; - MOV DPTR,#Zeile1		'Pointer auf die Zeichenkette setzen
; - CALL RS232_SEND_S		'Zeichenkette mit 0 Terminierung ausgeben
;
; - MOV A,#">"			'Schreibt ein Prompt (grsser als)
; - CALL RS232_SEND
;
;************************************************************************
;
; INCLUDE 89C2051.mc   


;89C2051 UART Konfig:
;Die in den Timer (das TH1 und in das TL1-Register) zu programmierenden Werte 
;berechnen sich wie folgt:
;                 K * Oszillatorfrequ.
;   Baud-Rate =  ------------------------
;                 32 * 12 * (256- TH1 ) 
; TH1 = 256-Quarz/192/Baud
;Wobei K = 1, wenn SMOD (PCON.7, Baudratenverdoppler) 0 ist.
;und K = 2 wenn SMOD = 1. 
;Genaue Baudraten sind freilich nur mit 'krummen' Quarzen wie 11,0592 MHz zu erreichen,
;aber die serielle Schnittstelle des PC ist nicht zimperlich -


;RS232_INIT    INITIALIZED 89C2051 SERIAL PORT
;              BAUD RATE: 9600
;              DATA LENGTH: 8 BIT
;              STOP BIT: 1 BIT
;              PARITY: NO
; 		   TH1,#0FEH @ X-TAL:  3.579545 MHz fr 9600BPs
; 		   TH1,#0FCH @ X-TAL:  8.00000  MHz fr 9600BPs
; 		   TH1,#0FAH @ X-TAL: 11,0592   MHz fr 9600BPs genau!!
; 		   TH1,#0F8H @ X-TAL: 14,3181   MHz fr 9600BPs
; 		   TH1,#0F7H @ X-TAL: 16,00000  MHz fr 9600BPs


RS232_INIT:    MOV       TMOD,#00100001B
               MOV       SCON,#01010010B
               MOV       PCON,#10000000B ; smod = 1
               MOV       TH1,#0F8H       ;TIMER1 LOAD VALUE
               SETB      TR1             ;START TIMER1
               RET


;RS232_SEND    SEND A TO SERIAL PORT
;              CHECK TRANSMITTER BUFFER BEFORE SEND
;
;              ENTRY: A
;              EXIT: NO
;
RS232_SEND:    JNB       TI,RS232_SEND
               CLR       TI
               MOV       SBUF,A          ;OK BUFFER EMPTY SEND OUT...
               RET


;RS232_GET     RECIVE BYTE FROM SERIAL PORT WITH TIME-OUT FUNCTION
;              ENTRY: NO
;              EXIT: A = ASCII NUMBER OR STRING COMMAND
;	       A = FF  TIME-OUT

RS232_GET:     MOV     R7,#05H
_RS232_AGAIN:  MOV     R6,#00H          ; TIME-OUT DELAY
_RS232_GET_RI: JB      RI,_RS232_READY

               DJNZ    R6,_RS232_GET_RI
               DJNZ    R7,_RS232_AGAIN
               MOV     A,#0FFH           ; TIME-OUT NO DATA RECEPTED
               RET

_RS232_READY:  CLR     RI
               MOV     A,SBUF
               RET


;RS232_GET_BW  RECIVE BYTE PUT TO A FROM SERIAL PORT
;              WAIT UNTIL RECIVER BUFFER READY
;              ENTRY: NO
;              EXIT: A

RS232_GET_BW:  JNB       RI,RS232_GET_BW
               CLR       RI
               MOV       A,SBUF      ; GET IT
               LCALL     RS232_SEND  ; ECHO TO TERMINAL
               RET


;RS232_CRLF    SEND CR TO RS232
;              ENTRY: NO
;              EXIT: NO
;
RS232_CRLF:    MOV       A,#0dh
               LCALL     RS232_SEND
               MOV       A,#0ah
               LCALL     RS232_SEND
               RET

;RS232_SPACE   SEND SPACE TO RS232
;              ENTRY: NO
;              EXIT: NO
;
RS232_SPACE:   MOV     A,#20H
               LCALL   RS232_SEND
               RET


;***************************************************************
; WriteHex      Hexadezimale Zeichenausgabe ber RS232
;
; Eingabeparameter : Zeichen im Akku
; Vernderte Reg.  : -
; Aufruf           : RS232_Send
;***************************************************************
RS232_Hex:	PUSH  ACC                ; Akku retten
		PUSH  ACC
		SWAP  A                  ; High Nibble zuerst
		CALL  _RS232Hex2Asc      ; konvertieren
		CALL  RS232_Send      
		POP   ACC                ; Akku zurck 
		CALL  _RS232Hex2Asc      ; Low Nibble
		CALL  RS232_Send      
		POP   ACC
		RET

_RS232Hex2Asc:	anl	a,#0fh		; Unteres Nibbel Abmskieren
		add	a,#246		; bertrag im Carry
		jnc	_RS232Hex2Asc2	; grsser 9
		add	a,#07		; 'A'-'9'
_RS232Hex2Asc2:	add	a,#58		; '9'+1
		RET




;RS232_SEND_S  SEND STRING CONSTANT TO TERMINAL
;              ENTRY: DPTR
;              EXIT: FOUND EOS
;
RS232_SEND_S:  CLR   A
               MOVC  A,@A+DPTR
               CJNE  A,#0,_RS232_SEND_S
               RET

_RS232_SEND_S: PUSH  DPL
               PUSH  DPH
               LCALL RS232_SEND
               POP   DPH
               POP   DPL
               INC   DPTR
               SJMP  RS232_SEND_S

